/*****************************************************************************/
/*                                                                           */
/* This file is part of a program which creates and manages a Microsoft .LIB */
/* format file containing Microsoft standard object modules. This version is */
/* specifically for use with IBM OS/2 2.0.                                   */
/*                                                                           */
/* Copyright (C) 1992 Jonathan Sykes (jsykes@aol.com)                        */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* General Public License for more details.                                  */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   File Name :  CreateDict.c                                               */
/*                                                                           */
/*   Called By :  Lib.c                                                      */
/*                                                                           */
/*   Calls     :  DictInsert.c, IsPrime.c                                    */
/*                                                                           */
/*   Date      :  29-Aug-92                                                  */
/*                                                                           */
/*****************************************************************************/
Word CreateDict (pdict)
DictPage *pdict[];
  {
  DictPage        *dict, *newdict;
  LibSym          *csym;
  Word             pages;
  int              safety = 30;
  int              i;

  NumSymbols = 0;
  SizSymbols = 0;

  for (csym = SymListHead; csym; csym = csym->Next)
    {
    NumSymbols++;
    SizSymbols += strlen (csym->Name);
    }

  NumSymbols += HASH_SIZE - 1;
  SizSymbols += SYM_BYTES_PAGE - 1;

  dict = NULL;
  while (dict == NULL)
    {
    safety += 15;
    pages   = max(NumSymbols * (100 + safety) / HASH_SIZE,
   	          SizSymbols * (100 + safety) / SYM_BYTES_PAGE) / 100;

    while (IsPrime(pages) == FALSE)
      pages++;

    dict  = (DictPage *)malloc(pages * sizeof(DictPage));

    for (i = 0; i < pages; i++)
      {
      memset (&dict[i], 0, sizeof(DictPage));
      dict[i].NFree = (HASH_SIZE + 1) / 2;
      }

    for (csym = SymListHead; csym; csym = csym->Next)
      if (DictInsert(dict, pages, csym) == FALSE)
        {
        free (dict);
        dict = NULL;
        break;
        }
    }

  *pdict = dict;

  return (pages);
  }
