

/* Output tables which say what global initializers need

   to be called at program startup, and what global destructors

   need to be called at program termination, for GNU C++ compiler.

   Copyright (C) 1987 Free Software Foundation, Inc.

   Hacked by Michael Tiemann (tiemann@mcc.com)

   COFF Changes by Dirk Grunwald (grunwald@flute.cs.uiuc.edu)

   OS/2 native mode version by Michael Johnson (michael.johnson@mixcom.com)



This file is part of GNU CC.



GNU CC is distributed in the hope that it will be useful,

but WITHOUT ANY WARRANTY.  No author or distributor

accepts responsibility to anyone for the consequences of using it

or for whether it serves any particular purpose or works at all,

unless he says so in writing.  Refer to the GNU CC General Public

License for full details.



Everyone is granted permission to copy, modify and redistribute

GNU CC, but only under the conditions described in the

GNU CC General Public License.   A copy of this license is

supposed to have been given to you along with GNU CC so you

can know your rights and responsibilities.  It should be in a

file named COPYING.  Among other things, the copyright notice

and this notice must be preserved on all copies.  */





/* This file contains all the code needed for the program `collect'.



   `collect' is run on all object files that are processed

   by GNU C++, to create a list of all the file-level

   initialization and destruction that need to be performed.

   It generates an assembly file which holds the tables

   which are walked by the global init and delete routines.

   The format of the tables are an integer length,

   followed by the list of function pointers for the

   routines to be called.



   Constructors are called in the order they are laid out

   in the table.  Destructors are called in the reverse order

   of the way they lie in the table.  */



#include "config.h"

#include <string.h>



int target_flags;	/* satisfy dependency in config.h */



#ifndef USE_COLLECT

main()

{

  exit(0);

}



#else





#include <sys/types.h>

#include <sys/stat.h>



#include <stdio.h>

/*#include "a_out.h"*/

#include "ar.h"



#ifdef UMAX

#include <sgs.h>

#endif



/*

 *	Define various output routines in terms on ASM_INT_OP,

 *	which should be defined in config.h -- if it's not, add it

 *	as the marker used to allocate an int on your architecture.

 */

#ifdef i386

#define ASM_INT_OP	ASM_LONG

/*#define NO_UNDERSCORES 1*/

#endif



#ifdef mips

#define NO_UNDERSCORES 1

/*

 * provide stubs for various things in tm-mips.h

 */

enum mips_sections cur_section;

int inside_function = 0;

int num_source_filenames = 0;

void print_options() { return; };

void data_section() { return; };

FILE *asm_out_text_file, *asm_out_data_file;

void pfatal_with_name(str)

char *str;

{

  fprintf(stderr,"collect: %s\n", str);

  exit(1);

}

void pic_label_is_global () {return;};

#endif



#ifndef ASM_OUTPUT_INT_CONST

#define ASM_OUTPUT_INT_CONST(FILE,VALUE)	\

  fprintf(FILE,"\t%s %d\n", ASM_INT_OP, VALUE)

#endif



#ifndef ASM_OUTPUT_LABELREF_AS_INT

#define ASM_OUTPUT_LABELREF_AS_INT(FILE,NAME)	\

  (fprintf(FILE,"\t%s ", ASM_INT_OP), \

   ASM_OUTPUT_LABELREF(FILE,NAME), \

   fprintf(FILE,"\n"))

#endif



#ifndef ASM_OUTPUT_PTR_INT_SUM

#define ASM_OUTPUT_PTR_INT_SUM(FILE,NAME,VALUE)	\

  (fprintf(FILE,"\t%s", ASM_INT_OP), \

   ASM_OUTPUT_LABELREF(FILE,NAME), \

   fprintf(FILE,"+%d\n", VALUE))

#endif



#ifndef ASM_OUTPUT_SOURCE_FILENAME

#define ASM_OUTPUT_SOURCE_FILENAME(FILE,NAME)\

  fprintf (FILE, "\t.file\t\"%s\"\n", NAME);

#endif



extern int xmalloc ();

extern void free ();



#ifndef CTOR_TABLE_NAME

#define CTOR_TABLE_NAME "__CTOR_LIST__"

#endif



#ifndef DTOR_TABLE_NAME

#define DTOR_TABLE_NAME "__DTOR_LIST__"

#endif



/*	Define or undef this in your config.h. Should be defined for

 *	MIPS & Sun-386i.

 */



#ifdef NO_UNDERSCORES

#  ifndef CTOR_DTOR_MARKER_NAME

#    ifndef NO_DOLLAR_IN_LABEL

#      define CTOR_DTOR_MARKER_NAME "_GLOBAL_$"

#    else

#      define CTOR_DTOR_MARKER_NAME "_GLOBAL_."

#    endif

#    define CTOR_DTOR_MARKER_LENGTH 9

#    define CTOR_DTOR_MARKER_OFFSET 0

#  endif

#else

#  ifndef CTOR_DTOR_MARKER_NAME

#    ifndef NO_DOLLAR_IN_LABEL

#      define CTOR_DTOR_MARKER_NAME "__GLOBAL_$"

#    else

#      define CTOR_DTOR_MARKER_NAME "__GLOBAL_."

#    endif

#    define CTOR_DTOR_MARKER_LENGTH 10

#    define CTOR_DTOR_MARKER_OFFSET 1

#  endif

#endif



enum error_code { OK, BAD_MAGIC, NO_NAMELIST,

		  FOPEN_ERROR, FREAD_ERROR, FWRITE_ERROR,

		  RANDOM_ERROR, };



enum error_code process ();

enum error_code process_a ();

int process_o ();

enum error_code output_ctor_dtor_table ();

void assemble_name ();



/* Files for holding the assembly code for table of constructor

   function pointer addresses and list of destructor

   function pointer addresses.  */

static FILE *outfile;



/* Default outfile name, or take name from argv with -o option.  */

static char *outfile_name = "a.out";



/* For compatibility with toplev.c and tm-sun386.h  */

char *dump_base_name;

int optimize = 0;

char *language_string = "GNU C++";

char *main_input_filename;



/*

 * The list of constructors & destructors. We process all files & then

 * spit these out in output_ctor_dtor_table(), because we need to know

 *  the length of the list.

 */



struct ctor_dtor_list_elem

{

  struct ctor_dtor_list_elem *next;

  char *name;

} *dtor_chain, *ctor_chain;



int dtor_chain_length = 0;

int ctor_chain_length = 0;



main (argc, argv)

     int argc;

     char *argv[];

{

  int i, nerrs = 0;

  enum error_code code;

  FILE *fp;

  char *infile;

  char *find_library(char *);



  if (argc > 2 && !strcmp (argv[1], "-o"))

    {

      outfile_name = argv[2];

      i = 3;

    }

  else

    i = 1;



  if ((outfile = fopen (outfile_name, "w")) == NULL)

    {

      perror ("collect");

      exit (-1);

    }



  dump_base_name = main_input_filename = outfile_name;

  for (; i < argc; i++)

    {

      char buf[80];



      if (argv[i][0] == '@') {

         char *response_file;

         FILE *rf;

         struct stat statbuf;

         int count = 0;

         char *rbuf;



         response_file = &argv[i][1];



         if ((rf = fopen (response_file, "rb")) == NULL)

         {

            sprintf (buf, "collect `%s'", response_file);

            perror (buf);

            exit (-1);

         }



         stat (response_file, &statbuf);

         rbuf = (char *) xmalloc (statbuf.st_size + 1);

         fread (rbuf, 1, statbuf.st_size, rf);

         rbuf[statbuf.st_size] = '\0';



         start_arg_list();



         while (*rbuf) {

            char c;

            char *argpos;



            while ((c = *rbuf) && (c == ' '

                                || c == '\t'

                                || c == '\n'

                                || c == 13)

                                || c == 26)

               ++rbuf;



            if (c) {

               argpos = rbuf;

               while ((c = *rbuf) && c != ' '

                                  && c != '\t'

                                  && c != '\n' 

                                  && c != 13

                                  && c != 26)

                  ++rbuf;

               save_arg (argpos, rbuf);

            }

         }



         build_argvec (&argc, &argv);

         i = -1;

         continue;

      }



      /* This is a library, find it.  */

      if (argv[i][0] == '-' && argv[i][1] == 'l')

	infile = find_library (&argv[i][2]);

      else

	infile = argv[i];



      if ((fp = fopen (infile, "rb")) == NULL)

	{

	  sprintf (buf, "collect `%s'", infile);

	  perror (buf);

	  exit (-1);

	}



      switch (code = process (fp, infile))

	{

	case OK:

	  break;



	case BAD_MAGIC:

	  fprintf (stderr, "file `%s' has a bad magic number for collect\n",

		   infile);

	  exit (-1);



	case NO_NAMELIST:

	  fprintf (stderr, "file `%s' has a no namelist for collect\n",

		   infile);

	  exit (-1);



	case RANDOM_ERROR:

	  fprintf (stderr, "random error while processing file `%s':\n",

		   infile);

	  perror ("collect");

	  exit (-1);



	case FOPEN_ERROR:

	  fprintf (stderr, "fopen(3S) error while processing file `%s' in collect\n",

		   infile);

	  exit (-1);



	case FREAD_ERROR:

	  fprintf (stderr, "fread(3S) error while processing file `%s' in collect\n",

		   infile);

	  exit (-1);



	case FWRITE_ERROR:

	  fprintf (stderr, "fwrite(3S) error while processing file `%s' in collect\n",

		   infile);

	  exit (-1);



	default:

	  abort ();

	}



      fclose (fp);



    }



  sift();

  collect();



  switch (code = output_ctor_dtor_table ())

    {

    case OK:

      fclose (outfile);

      exit (0);

    case FREAD_ERROR:

      perror ("fread(3S) failed in collect, at end");

      break;

    case FWRITE_ERROR:

      perror ("fwrite(3S) failed in collect, at end");

      break;

    case FOPEN_ERROR:

      perror ("fopen(3S) failed in collect, at end");

      break;

    case RANDOM_ERROR:

      fprintf (stderr, "random error in collect, at end");

      break;

    }

  exit (-1);

}



void

  add_ctor_dtor_elem(symbol_name)

char *symbol_name;

{

  /*

   *	In EXTENDED_COFF systems, it's possible to

   *	have multiple occurances of symbols ( or so it

   *	appears) in the symbol table. Sooo, we scan to

   *	eliminate duplicate entries. This can never hurt,

   *	and helps EXTENDED_COFF.

   */

  int exists;

  int is_ctor = (symbol_name[CTOR_DTOR_MARKER_LENGTH] == 'I');



  struct ctor_dtor_list_elem *p

    = ((is_ctor) ? ctor_chain : dtor_chain);



  exists = 0;

  while (p) {

    if (strcmp( symbol_name, p -> name ) == 0 ) {

      exists = 1;

      break;

    }

    p = p -> next;

  }

  if ( ! exists ) {



    struct ctor_dtor_list_elem *new = (struct ctor_dtor_list_elem *)

      xmalloc(sizeof(struct ctor_dtor_list_elem));



    new->name = (char *)xmalloc (strlen (symbol_name)

				 + CTOR_DTOR_MARKER_OFFSET + 2);



    strcpy(new -> name, symbol_name + CTOR_DTOR_MARKER_OFFSET);



    if ( is_ctor ) {

      new -> next = ctor_chain;

      ctor_chain = new;

      ctor_chain_length++;

    }

    else {

      new->next = dtor_chain;

      dtor_chain = new;

      dtor_chain_length++;

    }

  }

}





enum error_code

output_ctor_dtor_table ()

{

  int dtor_offset;



  /* Write out the CTOR tabel */

  ASM_FILE_START(outfile);



#ifdef EXTENDED_COFF

  fprintf (outfile, "%s\n", SDATA_SECTION_ASM_OP);

#else

  fprintf (outfile, "%s\n", DATA_SECTION_ASM_OP);

#endif

  ASM_GLOBALIZE_LABEL (outfile, CTOR_TABLE_NAME);

  ASM_OUTPUT_LABEL (outfile, CTOR_TABLE_NAME);

  ASM_OUTPUT_INT_CONST(outfile,ctor_chain_length);

  while ( ctor_chain ) {

    ASM_OUTPUT_LABELREF_AS_INT (outfile, ctor_chain->name);

    ctor_chain = ctor_chain -> next;

  }

  /* NULL-terminate the list of constructors.  -- not needed, but keep it */

  ASM_OUTPUT_INT_CONST (outfile, 0);



  /*

   * Now, lay out the destructors

   */



#ifdef EXTENDED_COFF

  fprintf (outfile, "%s\n", SDATA_SECTION_ASM_OP);

#else

  fprintf (outfile, "%s\n", DATA_SECTION_ASM_OP);

#endif

  ASM_GLOBALIZE_LABEL (outfile, DTOR_TABLE_NAME);

  ASM_OUTPUT_LABEL (outfile, DTOR_TABLE_NAME);

  ASM_OUTPUT_INT_CONST(outfile,dtor_chain_length);

  while (dtor_chain)

    {

      ASM_OUTPUT_LABELREF_AS_INT (outfile, dtor_chain->name);

      dtor_chain = dtor_chain->next;

    }

  ASM_OUTPUT_INT_CONST (outfile, 0);

  ASM_OUTPUT_INT_CONST (outfile, 0);



  fclose (outfile);

  return OK;

}





#ifdef OS2

/*****************************************************************************

 *	

 *		BSD SYMBOL TABLES

 *	

 *****************************************************************************/



/* Figure out the type of file we need to process.

   Currently, only .o and .a formats are acceptable.  */

enum error_code

process (fp, filename)

     FILE *fp;

     char *filename;

{

  struct stat file_stat;

  int size;

  unsigned char *buf;



  if (fstat (fp->_file, &file_stat))

    return RANDOM_ERROR;



  size = file_stat.st_size;



  buf = (char *)malloc(size);



  if (fread (buf, size, 1, fp) < 1)

    return RANDOM_ERROR;



  if ((int)buf[0] == (int)0xf0)

     return process_a(buf);

  else if ((int)buf[0] == (int)0x80) {

          process_o (buf, 0);

          return (0);

       }



  return BAD_MAGIC;

}



struct arg_list {

   char *arg;

   struct arg_list *next;

};



struct arg_list *arg_vec_list;



start_arg_list()

{

   arg_vec_list = NULL;

}



save_arg (char *bp, char *ep)

{

   int arglen = ep - bp;

   struct arg_list *arg_node;

   struct arg_list *p;



   if (arglen) {

      arg_node = (struct arg_list *) xmalloc (sizeof (struct arg_list));

      arg_node -> arg = (char *) xmalloc (arglen + 1);

      strncpy (arg_node -> arg, bp, arglen);

      arg_node -> arg[arglen] = '\0';

      arg_node -> next = NULL;



      if (p = arg_vec_list) {

         while (p -> next) p = p -> next;

         p -> next = arg_node;

      }

      else

         arg_vec_list = arg_node;

   }

}



build_argvec (int *argc, char **argv)

{

   int count;

   struct arg_list *p;

   char **targv;



   if (arg_vec_list == NULL) {

      *argc = -1;

      return;

   }



   for (count = 0, p = arg_vec_list; p; p = p -> next) ++count;



   targv = (char **) xmalloc ((count + 1) * sizeof (char *));



   *argv = (char *) targv;

   *argc = count;



   for (p = arg_vec_list; count; --count, p = p -> next)

      *targv++ = p -> arg;



   *targv = NULL;

}





char *

find_library (char *stub)

{

   char pathbuf[512];

   char namebuf[512];

   char *libloc;

   char *getenv();



   namebuf[0] = 'l';

   namebuf[1] = 'i';

   namebuf[2] = 'b';

   namebuf[3] = '\0';



   strcat (namebuf, stub);

   strcat (namebuf, ".lib");



   if (libloc = getenv ("LIB")) {

      struct stat statbuf;



      while (*libloc == ' ') ++libloc;

      while (*libloc) {

         int i = 0;

         while (*libloc && *libloc != ';') pathbuf[i++] = *libloc++;

         pathbuf[i] = '\0';

         strcat (pathbuf, "\\");

         strcat (pathbuf, namebuf);

         if (stat (pathbuf, &statbuf) == 0)

            return (strcpy ((char *)xmalloc (strlen (pathbuf) + 1), pathbuf));

         if (*libloc == ';') ++libloc;

      }

   }



   return (namebuf);

}





struct name_record {

   struct name_record *next;

   char *name;

};



struct object {

   struct object *next;

   struct object *prev;

   struct name_record *EXTDEFs;

   struct name_record *PUBDEFs;

};



struct object *library_objects = 0;

struct object *needed_objects = 0;

struct object *current_object = 0;



start_object (int fromlib)

{

   struct object *new_object;



#ifdef DEBUG

   fprintf (stderr, "Starting a new object\n");

#endif



   new_object = (struct object *)malloc (sizeof (struct object));



   if (fromlib) {

      new_object -> next = library_objects;

      new_object -> prev = 0;

      if (library_objects)

         library_objects -> prev = new_object;

      library_objects = new_object;

   } else {

      new_object -> next = needed_objects;

      new_object -> prev = 0;

      if (needed_objects)

         needed_objects -> prev = new_object;

      needed_objects = new_object;

   }



   current_object = new_object;

}





record_EXTDEF (char * symbol)

{

   struct name_record *new_name;



#ifdef DEBUG

   fprintf (stderr, "recording EXTDEF \'%s\'\n", symbol);

#endif



   new_name = (struct name_record *)malloc (sizeof (struct name_record));



   new_name -> name = strdup (symbol);

   new_name -> next = current_object -> EXTDEFs;

   current_object -> EXTDEFs = new_name;

}





record_PUBDEF (char * symbol)

{

   struct name_record *new_name;



#ifdef DEBUG

   fprintf (stderr, "recording PUBDEF \'%s\'\n", symbol);

#endif



   new_name = (struct name_record *)malloc (sizeof (struct name_record));



   new_name -> name = strdup (symbol);

   new_name -> next = current_object -> PUBDEFs;

   current_object -> PUBDEFs = new_name;

}





int search_object (struct object *to_search, char *to_find)

{

   struct name_record *name_list = to_search -> PUBDEFs;



   while (name_list) {

      if (strcmp (name_list -> name, to_find) == 0)

         return (1);



      name_list = name_list -> next;

   }



   return (0);

}



struct object *

search_library (char *to_find)

{

   struct object *object_list = library_objects;



#ifdef DEBUG

   fprintf (stderr, "Searching libraries for %s\n", to_find);

#endif



   while (object_list) {

      if (search_object (object_list, to_find))

#ifdef DEBUG

      {

         fprintf (stderr, "Found %s in libraries\n", to_find);

#endif

         return (object_list);

#ifdef DEBUG

      }

#endif



      object_list = object_list -> next;

   }



#ifdef DEBUG

   fprintf (stderr, "Couldn't find %s in libraries\n", to_find);

#endif



   return (0);

}





append_needed(struct object *needed_object)

{

   struct object *object_list = needed_objects;



   while (object_list -> next)

      object_list = object_list -> next;



   /* unlink from library_objects */



   if (needed_object -> prev)

      /* not first */

      needed_object -> prev -> next = needed_object -> next;

   else

      /* first */

      library_objects = needed_object -> next;



   if (needed_object -> next)

      /* not last */

      needed_object -> next -> prev = needed_object -> prev;



   /* append to needed_objects */



   while (object_list -> next) object_list = object_list -> next;

   object_list -> next = needed_object;

   needed_object -> prev = object_list;

   needed_object -> next = 0;

}





sift()

{

   struct object *needed = needed_objects;

   struct name_record *names;

   char *looking_for;

   struct object *found_def;



   while (needed) {

      names = needed -> EXTDEFs;

      while (names) {

         looking_for = names -> name;



#ifdef DEBUG

         fprintf (stderr, "***sifting for %s\n", looking_for);

#endif

         if (!search_object (needed, looking_for))

            if (found_def = search_library(looking_for)) {

               append_needed (found_def);

               continue;

            }



         names = names -> next;

      } /* endwhile */



      needed = needed -> next;

   }

}





collect()

{

   struct object *object_list = needed_objects;

   struct name_record *name_list;



   while (object_list) {

      name_list = object_list -> EXTDEFs;

      while (name_list) {

         if (strncmp (name_list -> name, "__GLOBAL_$", 10) == 0)

            add_ctor_dtor_elem (name_list -> name);



         name_list = name_list -> next;

      }



      object_list = object_list -> next;

   } /* endwhile */

}





#define MODEND          0x8A

#define END386          0x8B

#define EXTDEF          0x8C

#define PUBDEF          0x91

#define COMDEF          0xB0

#define LOCEXD          0xB4

#define MARKER          0xF1



int

process_o (unsigned char *buf, int fromlib)

{

  char *next_record;

  char symbol[256];

  int namelength;



  start_object(fromlib);



  while (1) {



     next_record = buf + (buf[1] + (buf[2] << 8) + 3);



     switch (buf[0]) {



      case EXTDEF:

        buf += 3;

        namelength = *buf++;

        strncpy (symbol, buf, namelength);

        symbol[namelength] = '\0';

        record_EXTDEF(symbol);

        buf = next_record;

        break;



      case PUBDEF:

        buf += 5;

        namelength = *buf++;

        strncpy (symbol, buf, namelength);

        symbol[namelength] = '\0';

        record_PUBDEF(symbol);

        buf = next_record;

        break;



      case (int)MODEND:

      case (int)END386:

        buf = next_record;

        return (buf);



     default:

        buf = next_record;

        break;



     } /* endswitch */

  } /* endwhile */

  return OK;

}



enum error_code

process_a (unsigned char *buf)

{

  char *next_record;

  char temp[256];



  buf += 16;



  while ((int)buf[0] != (int)MARKER) {



     next_record = buf + (buf[1] + (buf[2] << 8) + 3);



     switch ((int)buf[0]) {



      case (int)0x80:

        buf = process_o (buf, 1);

        while (*buf == '\0') buf++;

        break;



      case 0:

        buf += 16;

        break;

/*

      case (int)MODEND:

        buf = next_record;

        while (*buf == '\0') buf++;

        break;

*/

      default:

        fprintf (stderr, "collect: Error processing library\n");

        exit (1);

        buf = next_record;

        break;



     } /* endswitch */

  } /* endwhile */

  return OK;

}

#endif

/* Output to FILE a reference to the assembler name of a C-level name NAME.

   If NAME starts with a *, the rest of NAME is output verbatim.

   Otherwise NAME is transformed in an implementation-defined way

   (usually by the addition of an underscore).

   Many macros in the tm file are defined to call this function.



   Swiped from `varasm.c'  */



void

assemble_name (file, name)

     FILE *file;

     char *name;

{

  if (name[0] == '*')

    fputs (&name[1], file);

  else

#ifdef NO_UNDERSCORES

    fprintf (file, "%s", name);

#else

    fprintf (file, "_%s", name);

#endif

}



int

xmalloc (size)

     int size;

{

  int result = malloc (size);

  if (! result)

    {

      fprintf (stderr, "Virtual memory exhausted\n");

      exit (-1);

    }

  return result;

}

#endif

